@file:JvmName("Compilations")

package ru.landgrafhomyak.kotlin.kmp_gradle_build_helper

import org.jetbrains.kotlin.gradle.dsl.KotlinCommonCompilerOptions
import org.jetbrains.kotlin.gradle.dsl.KotlinCommonOptions
import org.jetbrains.kotlin.gradle.dsl.KotlinMultiplatformExtension
import org.jetbrains.kotlin.gradle.plugin.KotlinCompilation
import org.jetbrains.kotlin.gradle.dsl.KotlinVersion

public fun KotlinMultiplatformExtension.configureAllCompilations(action: KotlinCompilation<KotlinCommonOptions>.() -> Unit) {
	this.targets.configureEach { t ->
		t.compilations.configureEach(action)
	}
}

public fun KotlinMultiplatformExtension.configureAllCompilersOptions(action: KotlinCommonCompilerOptions.() -> Unit) {
	this.targets.configureEach { t ->
		t.compilations.configureEach { c ->
			c.compileTaskProvider.configure { t ->
				t.compilerOptions(action)
			}
		}
	}
}

public fun KotlinMultiplatformExtension.noWarnExpectActual(): Unit =
	this.configureAllCompilersOptions { this@configureAllCompilersOptions.freeCompilerArgs.add("-Xexpect-actual-classes") }

public fun KotlinMultiplatformExtension.warningsAsErrors(): Unit =
	this.configureAllCompilersOptions { this@configureAllCompilersOptions.allWarningsAsErrors.set(true) }

public fun KotlinMultiplatformExtension.optIn(classQualname: String): Unit =
	this.configureAllCompilersOptions { this@configureAllCompilersOptions.optIn.add(classQualname) }

public fun KotlinMultiplatformExtension.optInContracts(): Unit =
	this.optIn("kotlin.contracts.ExperimentalContracts")

public fun KotlinMultiplatformExtension.setCompatibilityWithKotlin(version: KotlinVersion): Unit =
	this.configureAllCompilersOptions {
		this@configureAllCompilersOptions.apiVersion.set(version)
		this@configureAllCompilersOptions.languageVersion.set(version)
	}